unit QRepBarcode;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, QuickRpt, QRCtrls, Graphics;

type
  TCodeType = (EAN_8, EAN_13, UPC_A, UPC_E, CODE39, CODE25i, CODE25iP,
    CODE128_A, CODE128_B, CODE128_C, MSI_Plessey, Codabar, Postnet,
    Postnet_FIM);
  TModuleWidth = 0..9;
  THeightPercent = 20..200;
  TSaveToFileProc = procedure(BCType: TCodeType; Module: TModuleWidth;
    HeightPercent: THeightPercent; Text, FileName: PChar);
  TSaveToStreamProc = procedure(BCType: TCodeType; Module: TModuleWidth;
    HeightPercent: THeightPercent; Text: PChar; Dest: Pointer; var Size: Int64);

  TQRepBarcode = class(TQRImage)
  private
    FModuleWidth: TModuleWidth;
    FCodeType: TCodeType;
    FCaption: string;
    FFont: TFont;
    FHeightPercent: THeightPercent;
    procedure SetFont(const Value: TFont);
    procedure DoText(BMP: TBitmap);
    procedure SetCodeType(const Value: TCodeType);
    procedure ReloadBarcode;
    procedure LoadBarcode;
    procedure SetCaption(const Value: string);
    function SetHeightPercent: THeightPercent;
  protected
    procedure Prepare; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property ModuleWidth: TModuleWidth read FModuleWidth write FModuleWidth;
    property CodeType: TCodeType read FCodeType write SetCodeType;
    property Caption: string read FCaption write SetCaption;
    property Font: TFont read FFont write SetFont;
    property HeightPercent: THeightPercent read SetHeightPercent write
      FHeightPercent;
  end;

const
  CodeTypeStr: array[TCodeType] of string = ('EAN 8', 'EAN 13',
    'UPC A', 'UPC E', 'CODE 39', 'CODE 25i', 'CODE 25iP',
    'CODE 128 A', 'CODE 128 B', 'CODE 128 C', 'MSI', 'Codabar', 'Postnet',
    'Postnet FIM');

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TQRepBarcode]);
end;

{ TQRBarcode }

constructor TQRepBarcode.Create(AOwner: TComponent);
begin
  inherited;
  FFont := TFont.Create;
  FModuleWidth := 2;
  FHeightPercent := 100;
  FCodeType := UPC_E;
  FCaption := '123456'
end;

procedure TQRepBarcode.DoText(BMP: TBitmap);
var
  W, H, BH: Integer;
begin
  if FFont.Size <> 0 then
  begin
    BMP.Canvas.Font := FFont;
    BMP.Monochrome := True;
    W := BMP.Canvas.TextWidth(FCaption);
    H := BMP.Canvas.TextHeight(FCaption);
    BH := BMP.Height;
    BMP.Height := BH + H - 2;
    BMP.Canvas.TextOut((BMP.Width - W) div 2, BH - 2, FCaption);
  end;
  Picture.Assign(BMP);
end;

procedure TQRepBarcode.LoadBarcode;
var
  H: THandle;
  Proc: TSaveToStreamProc;
  Bmp: TBitmap;
  MS: TMemoryStream;
  Buffer: array[0..131071] of Byte; //128 KB sollten ausreichen...
  Size: Int64;
begin
  Picture.Assign(nil);
  Stretch := False;
  AutoSize := True;
  H := LoadLibrary('BC.DLL');
  if H <> 0 then
  begin
    @Proc := GetProcAddress(H, 'SaveToStream');
    if @Proc <> nil then
    begin
      Proc(FCodeType, FModuleWidth, FHeightPercent, PChar(FCaption), @Buffer, Size);
      MS := TMemoryStream.Create;
      MS.WriteBuffer(Buffer, Size);
      Bmp := TBitmap.Create;
      MS.Seek(0, 0);
      Bmp.LoadFromStream(MS);
      MS.Free;
      Dotext(Bmp);
      Bmp.Free;
    end;
    FreeLibrary(H);
  end;
end;

procedure TQRepBarcode.Prepare;
begin
  inherited;
  LoadBarcode;
end;

procedure TQRepBarcode.ReloadBarcode;
begin
  if not (csDesigning in ComponentState) then
    LoadBarcode;
end;

procedure TQRepBarcode.SetCaption(const Value: string);
begin
  FCaption := Value;
  ReloadBarcode
end;

procedure TQRepBarcode.SetCodeType(const Value: TCodeType);
begin
  FCodeType := Value;
  ReloadBarcode
end;

procedure TQRepBarcode.SetFont(const Value: TFont);
begin
  if FFont <> Value then
    FFont.Assign(Value);
  ReloadBarcode
end;

function TQRepBarcode.SetHeightPercent: THeightPercent;
begin
  Result := FHeightPercent;
  ReloadBarcode
end;

end.

